from math import exp
from numpy import mat, shape, ones


def sigmoid(inX):
    return 1.0 / (1 + exp(-inX))


def gradAscent(dataMatIn, labels, 
               alpha = 0.01, maxCycles = 20):
  """
  count the best weights.
  """

  dataMatrix =  mat(dataMatIn)
  labelMatrix = mat(labels).transpose()

  m,n = shape(dataMatrix)

  weights = ones((n, 1))

  for k in range(maxCycles):

    h = sigmoid(sum(dataMatrix * weights))

    error = (labelMatrix - h )
    weights = weights + alpha * dataMatrix.transpose() * error

  return weights


def classify(inX, weights):
  prob = sigmoid(inX * weights)
  return 1 if prob > 0.5 else 0




import operator

from numpy import tile


def classify(inX, dataset, labels, k):
    """
        inX: the dataset you want to predict
        dataset: the training dataset attribute vector
        labels: the labels of training dataset
        k: n
    """
    # size: the size of dataset
    size = dataset.shape[0]

    # copy x to count the diff
    diff_mat = tile(inX, (size, 1)) - dataset
    sq_diff_mat = diff_mat ** 2
    sq_distances = sq_diff_mat.sum(axis=1)
    distances = sq_distances ** 0.5

    # sort distance, and index as the value of new array
    sorted_distances = distances.argsort()


    # dict contains the votes
    count = {}
    for i in range(k):
        label_index = labels[sorted_distances[i]]
        count[label_index] = count.get(label_index, 0) + 1
    # sort by votes
    sorted_count = sorted(count.items(),
                          key=operator.itemgetter(1),
                          reverse=True)

    return sorted_count[0][0]


from numpy import zeros, shape, tile

def normalize(dataset):
  """
  dataset: the dataset will be normalized
  return:
    retDataset: normalized dataset
    minVals: the minimum values  vector
    ranges: the difference of minVals and maxVals
  """
  minVals = dataset.min(0)
  maxVals = dataset.max(0)
  ranges = maxVals - minVals

  retDataset = zeros(shape(dataset))

  m = dataset.shape[0]
  retDataset = dataset - tile(minVals, (m, 1))
  retDataset = retDataset / tile(ranges, (m, 1))

  return retDataset, minVals, ranges 




def train_test_split(dataset, labels, ratio):
  size  = len(dataset)
  sep = int((size-1) * ratio)
  # train_x, train_y, test_x, test_y
  return dataset[:sep], labels[:sep], \
         dataset[sep:], labels[sep:]

def overSampling(dataset, labels, aimedLabel , times):
  """
  for aimedLabel overSampling n times.
  """
  new_x = []
  new_y = []

  idx = 0
  for label in labels:
    if label == aimedLabel:
      for  i in range(times):
         new_x.append(dataset[idx])
         new_y.append(dataset[idx])
    idx += 1

  return new_x, new_y

def underSampling(dataset, labels, aimedLabel, times):
  new_x = []
  new_y = []
  cnt = 0
  idx = 0
  for label in labels:
    if label == aimedLabel:
      cnt +=1
      if cnt >= times:
        new_x.append(dataset[idx])
        new_y.append(label)
    idx += 1
  
  return new_x, new_y



from distutils.core import setup

setup(
    name='mlkit_lyt',
    packages=['mlkit_lyt', 'mlkit_lyt.KNN', 'mlkit_lyt.logistic', 'mlkit_lyt.common'],
    author='lyt0628',
    author_email='lyt.0628@qq.com',
    url='http://lyt0628.icu',
    download_url='http://github.com//LYT0628/mllearn',
    description='Machine Learning in Python'
)
